==> linuxconf-tools-1.1.src.tar.gz: linuxconf-tools-1.1/projetx/ufusion.cc <==
#include <string.h>
#include <misc.h>
#include <dialog.h>
#include "projet.h"
#include "projetx.m"

/* #Spcification: Fusion / Espace usager
	Un usager peut continuer  dvelopper pendant qu'une
	intgration a lieu. Aprs l'intgration, il souhaitera
	rappatrier ces changements dans la nouvelle rvision.

	(histoire de travailler avec des bits frais: Il est
	reconnue que la fraicheur des bits est inversement
	proportionnelle  l'age du capitaine. C'est pourquoi
	il faudra voter non le 26 octobre prochain)

	Voici un exemple:

		L'exemple suivant est purement fictif et toute
		ressemblance avec la ralit est purement
		le fruit du hasard.

		Dans une compagnie fictive MSK, il y a trois
		programmeurs: Woodbang dit mosart, Jambon dit bambin,
		et louis dit le casseur de rseau.

		Tous trois sont occups  amliorer la version 4.2
		du programme 69/69. Et soudain avant mme que l'automne
		ne fut venue, chacun fait une livraison. Et il continue
		ensuite  travailler dans leur version respective.

		Woodbang livre la version 4.2.4.1
		Jambon livre la version 4.2.3.1
		Louis livre la version 4.2.6.1

		Pierrot le loup fait une intgration  partir des
		versions 4.2 4.2.3.1 4.2.4.1 4.2.6.1. Cela donne
		la version 4.3. Elle se compose de:

			Majoritairement des fichiers de 4.2.
			Des fichiers provenant d'une des 3 rvisions.
			Des fichiers provenant de la combinaison
				des 3 rvisions.

		Lorsque Woodbang dcide de travailler sur la
		version 4.3 et ainsi profiter des nouvelles options
		auto-reboot de louis le casseur et Jambon, quelques minutes
		se sont pass depuis qu'il a livr sa version 4.2.4.1. Entre
		temps, il a ajout 25 nouvelles options au programme. Ces
		options ont t ajouts  la rvision 4.2.4.1. Il ne peut
		pas simplement copier les fichiers modifis dans sa copie
		de la version 4.3.

	La rvision dans laquelle il a continu  travailler est
	la rvision "temporaire". La rvision dans laquelle il
	souhaite incorporer ses changement est la rvision destination.

*/
#

/*
	Enregistre conditionnellement les transactions pour faire l'intgration
	d'un fichier.
*/
static void ufusion_settrans(
	INTEGRE_DAT &intg,
	USERINFO *userref,
	MAKEFILE &mkfref,		// Rfrence commune approxim.
	USERINFO *usersrc,
	MAKEFILE &mkfsrc,		// Provenance immdiate du fichier rvis.
	USERINFO *userdst,
	MAKEFILE &mkfdst,		// Destination de l'intgration.
	const char *dirsrc,		// Path du rpertoire projet qui contient
							// fichier.
	const char *fichier,
	int efface)				// Ce fichier a t effac de la rvision source
{
	/* #Spcification: fusion / espace usager / rvision bidon
		Les fichiers qui sont dans l'espace usager n'ont pas
		encore t livrs. On va leur attribu une rvision
		bidon facile  reconnaitre lors de l'intgration.

		On placera simplement comme rvision dans integre.dat
		le numro 1000.0.0.0. Le module d'dition de la fusion fera
		un cas spcial pour ce numro de rvision.
	*/
	REVISION spcrev;	// Le constructeur assure 
						// spcrev.isnull() != 0

	const REVISION *revroot_src = usersrc->getrevsrc();
	const REVISION *revroot_dst = userdst->getrevsrc();

	MAKEFILE_FILE *fsrc = mkfsrc.locate(fichier);
	MAKEFILE_FILE *fdst = mkfdst.locate(fichier);
	MAKEFILE_FILE *fref = mkfref.locate(fichier);
	char srcpath[MAXSIZ_PATH]; // Path du fichier non livr
	path_make (dirsrc,fichier,srcpath);
	if (fsrc != NULL
		&& fdst != NULL
		&& fsrc->getrev()->cmp(fdst->getrev()) == 0){
		// ICI on est certain que la rvision usersrc est l'anctre
		// de userdst et videmment de la rvision  fusionner.
		char pathref[MAXSIZ_PATH];
		fsrc->getombpath(usersrc,pathref);
		FILE_REV_SPEC ref;
		ref.fichier  = fsrc->getrev();
		ref.dir = mkfsrc.getrev();
		ref.root = revroot_src;
		/* #Spcification: Fusion / Espace usager / automatique
			Pour importer sans intervention un fichier
			d'une rvision temporaire vers la rvision
			destination, il faut que les deux
			rvisions possdent la mme version de ce fichier.
		*/
		FILE_REV_SPEC rev;
		rev.fichier  = &spcrev;
		rev.dir = ref.dir;
		rev.root = ref.root;
		intg.addtrans(fichier,ref
			,pathref,efface ? '-' : '+'
			,rev,srcpath);
	}else{
		// ICI, on prend userref comme rvision anctre. C'est une
		// approximation. Se referer  la spec plus bas pour comprendre
		// les noms des variables ref, tmp et dst
		FILE_REV_SPEC ref;
		ref.fichier = NULL;
		ref.dir = mkfref.getrev();
		ref.root = userref->getrevsrc();
		FILE_REV_SPEC tmp;
		tmp.fichier = NULL;
		tmp.dir = mkfsrc.getrev();
		tmp.root = revroot_src;
		FILE_REV_SPEC dst;
		dst.fichier = NULL;
		dst.dir = mkfdst.getrev();
		dst.root = revroot_dst;

		/* #Spcification: Fusion / Espace usager / manuelle
			Voici un exemple, o une fusion interactive est requise.
			Soit le fichier A et ses diffrentes rvisions:

			#
			      ---> 4.2.3.1 -------
				 |				      |--> 4.3 .....................
			4.2--|                 ---	   (Rvision destination)   .
				 |				  |									.
				  ---> 4.2.4.1 -->|									.
								   --- Rvision temporaire	........
									   (4.2.4.2)
			4.2 est la rvision rfrence
			#
			Pour simplifier la fusion, l'usager doit
			pouvoir visualiser les diffrences suivantes:
			#
				4.2.4.1 vs 4.2
				4.2.3.1 vs 4.2
				4.3     vs 4.2
				4.2.4.2 vs 4.2.4.1 (Probablement
									la plus importante).
			#

			La version 4.3 a t construites manuellement  partir
			de diffrentes versions (4.1.3.1 et 4.1.4.1). De plus
			l'intgrateur a pu ajout d'autre chose (l'ivresse du moment).
			La provenance de la version 4.3 ne peut pas tre dmontr absoluement.
			C'est pourquoi, il faut permettrent  peu prs toutes les
			combinaisons de diffrences imaginables.
		*/
		char *pathref = NULL;
		char tmppath[MAXSIZ_PATH];
		if (fref != NULL){
			ref.fichier = fref->getrev();
			fref->getombpath(userref,tmppath);
			pathref = tmppath;
		}
		if (efface){
			intg.addtrans(fichier,ref,pathref
				,'-'
				,tmp,srcpath);
		}else{
			tmp.fichier = &spcrev;
			intg.addtrans(fichier,ref,pathref
				,fsrc == NULL && fdst == NULL ? '?' : '+'
				,tmp,srcpath);
		}
		if (fsrc != NULL){
			char tmppath[MAXSIZ_PATH];
			fsrc->getombpath(usersrc,tmppath);
			tmp.fichier = fsrc->getrev();
			intg.addtrans (fichier,ref
				,pathref,'+'
				,tmp,tmppath);
		}else{
			intg.addtrans (fichier,ref
				,pathref,'-'
				,tmp,NULL);
		}
		if (fdst != NULL){
			char tmppath[MAXSIZ_PATH];
			fdst->getombpath(userdst,tmppath);
			dst.fichier = fdst->getrev();
			intg.addtrans (fichier,ref
				,pathref,'+'
				,dst,tmppath);
		}else{
			intg.addtrans (fichier,ref
				,pathref,'-'
				,dst,NULL);
		}
	}
}

/*
	Balaye les deux rpertoires rcursivement et construit un integre.dat
	dans chaque rpertoire. Ces integre.dat agira comme guide pour
	la fusion. Mme format que pour les fusions dans /kit/build.
*/
static int ufusion_scan (
	USERINFO *usersrc,		// Environnement  intgrer dans userdst
	USERINFO *userdst,
	USERINFO *userref,		// Racine des deux environnement prcdant
	PRJCTRL *ctrl,
	const char *projet,
	PROJET_LOG *log,
	MAKEFILE *mkrefanc,		// makefile.dat du projet anctre de celui-ci
							// obtenue  partir de userref.
							// == NULL si projet == ""
	MAKEFILE *mksrcanc)		// makefile.dat du projet anctre de celui-ci
							// obtenue  partir de usersrc.
							// == NULL si projet == ""
	
{
	char dirdst[MAXSIZ_PATH];
	userdst->makusrpath(projet,dirdst);
	/* #Spcification: fusion / Espace usager / slection des rpertoires.
		La fusion est faite uniquement pour les projets que l'usager
		a import pour la rvision cible.
	*/
	if (file_type(dirdst)==1){
		char dirsrc[MAXSIZ_PATH];
		usersrc->makusrpath(projet,dirsrc);
		char *tb[200];
		int nb = ctrl->dirsource (dirsrc,WILD_ALLFILE,tb,200);
		log->printf ("Rpertoire %s [%d]\n",dirsrc,nb);
		if (nb > 0){
			/* #Spcification: fusion / espace usager / rvision rfrence
				La rvision rfrence n'est probablement pas
				disponible dans l'espace usager. Pour un
				projet ou sous-projet, on doit retrouver la rvision
				correspondante. En effet la rvision X est une combinaison
				de rvisions diffrentes de sous-projet.
			*/
			const REVISION *revision_ref = userref->getrevsrc();
			const REVISION *revision_src = usersrc->getrevsrc();
			if (projet[0] != '\0'){
				// Sous-projet, il faut consulter mkrefanc et mksrcanc pour
				// dterminer la bonne rvision rfrence et source.
				char nom[MAXSIZ_NAME*2];
				path_splitlex (projet,NULL,nom);
				strcat (nom,"/makefile.dat");
				MAKEFILE_FILE *file = mkrefanc->locate(nom);
				if (file == NULL){
					// Nouveau rpertoire.
					log->printf ("Nouveau rpertoire %s\n",projet);
					/* #Spcification: fusion / espace usager / nouveaux rpertoires
						Le traitement  raliser n'est pas clair
						dans le cas ou un nouveau rpertoire ou projet
						a t cr. En ce moment, on signale simplement
						 l'usager qui devra fusionner manuellement le
						rpertoire en question.
					*/
					// On laisse revision_ref comme il est.
					// En utilisant la rvision de la racine
					// ca va charger un makefile.dat vide.
				}else{
					revision_ref = file->getrev();
				}
				// Mme traitement pour la rvision source
				file = mksrcanc->locate(nom);
				if (file == NULL){
					// Nouveau rpertoire.
					log->printf ("Nouveau rpertoire %s : environnement tmp\n",projet);
					// On laisse revision_src comme il est.
					// En utilisant la rvision de la racine
					// ca va charger un makefile.dat vide.
				}else{
					revision_src = file->getrev();
				}
			}
			MAKEFILE mkfref(NULL,1,revision_ref,projet,userref);
			// Pour le rpertoire source, on obtient le makefile.dat
			// de /kit/ombre, pour eviter le makefile.del locale.
			// Voir MAKEFILE::MAKEFILE
			MAKEFILE mkfsrc(NULL,1,revision_src,projet,usersrc);
			// Scan les sous-rpertoires
			for (int i=0; i<nb; i++){
				char *fichier = tb[i];
				if (ctrl->isdir(fichier)){
					char sub_projet[MAXSIZ_PATH];
					path_make (projet,fichier,sub_projet);
					path_stripsep(sub_projet,sub_projet);
					ufusion_scan (usersrc,userdst,userref,ctrl
						,sub_projet,log,&mkfref,&mkfsrc);
				}
			}
			MAKEFILE mkfdst(NULL,1,projet,userdst);
			INTEGRE_DAT intg(dirdst);
			for (int i=0; i<nb; i++){
				char *fichier = tb[i];
				if (!ctrl->isdir(fichier)){
					ufusion_settrans (intg,userref,mkfref,usersrc,mkfsrc
						,userdst,mkfdst,dirsrc,fichier,0);
				}
			}
			/* #Spcification: Fusion / espace usager / makefile.del
				Les fichiers makefile.del sont interprter lors de la
				fusion et prsenter interactivement lors de l'dition
				de la fusion.
			*/
			char delpath[MAXSIZ_PATH]; // Path du fichier makefile.del
			path_make (dirsrc,"makefile.del",delpath);
			FILE *fdel = fopen (delpath,"r");
			// Le fichier makefile.del est optionnel.
			if (fdel != NULL){
				char buf[MAXSIZ_PATH];
				while (fgets(buf,sizeof(buf)-1,fdel)!=NULL){
					str_strip (buf,buf);
					if (buf[0] != '\0'){
						ufusion_settrans (intg,userref,mkfref,usersrc,mkfsrc
							,userdst,mkfdst,dirsrc,buf,1);
					}
				}
				fclose (fdel);
			}			
		}
		tbstr_free (tb,nb);
	}
	return 0;
}
/*
	Slectionne deux rvisions  fusionner dans l'environnement usager
	et produit les fichiers integre.dat pilotant la fusion interactive
	de integre_do().
*/
void ufusion_prepare (USERINFO *user)
{
	char home[MAXSIZ_PATH];
	user->gethome(home);
	char *tbrev[100];
	int nb = projet_getrevdir (home,tbrev,100);
	if (nb < 2){
		xconf_error (MSG_U(E_MIN2PRJ
			,"Vous devez avoir au moins deux rvisions"));
	}else{
		DIALOG dia;
		char src = 0;
		char dst = 1;
		dia.newf_head ("",MSG_U(H_FUSIONSIMPLE
			,"Tmp\tDst\tprog/Version/rpertoire/source"));
		for (int i=0; i<nb; i++){
			char revstr[MAXSIZ_REVISION];
			revision_path2rev (tbrev[i],revstr);
			#if 0
			dia.newf_radio (&src,i,i==0);
			page_setupradioc (&page,-1,&dst,i,i==1);
			page_setupfixestr0 (&page,-1,revstr,MAXSIZ_REVISION);
			#endif
		}
		int nof =0;
		MENU_STATUS code = dia.edit (MSG_U(T_FUSIONSIMPLE,"Fusion simple")
			,""
			,help_nil
			,nof);
		if (code == MENU_ACCEPT){
			USERINFO usersrc(user);
			char verstr[MAXSIZ_REVISION];
			revision_path2rev(tbrev[src],verstr);
			usersrc.setversion(verstr);
			/* #Spcification: fusion / espace usager / rvision rfrence
				La rvision rfrence (commune  la rvision temporaire
				et la destination) est la version d'intgration
				de la rvision temporaire.
			
					Rvision temporaire : 4.2.4.1
					Rvision destination: 4.3
					Rvision rfrence  : 4.2

				C'est un approximation. Il n'est pas toujours possible
				de dterminer la rvision rfrence (anctre) exacte
				de deux rvision.
			*/
			REVISION rev(verstr);
			rev.cnv (REVISION_INTEGRE);
			rev.format (verstr);
			USERINFO userref(user);
			userref.setversion(verstr);
			USERINFO userdst(user);
			revision_path2rev(tbrev[dst],verstr);
			userdst.setversion(verstr);
			PRJCTRL ctrl (user);
			WINDOW_LOG log(NULL,1);
			log.show();
			ufusion_scan(&usersrc,&userdst,&userref,&ctrl,"",&log
				,NULL,NULL);
		}
	}
}

==> linuxconf-tools-1.7.src.tar.gz: linuxconf-tools-1.7/projetx/ufusion.cc <==
#include <string.h>
#include <misc.h>
#include <dialog.h>
#include "projet.h"
#include "projetx.m"

/* #Spcification: Fusion / Espace usager
	Un usager peut continuer  dvelopper pendant qu'une
	intgration a lieu. Aprs l'intgration, il souhaitera
	rappatrier ces changements dans la nouvelle rvision.

	(histoire de travailler avec des bits frais: Il est
	reconnue que la fraicheur des bits est inversement
	proportionnelle  l'age du capitaine. C'est pourquoi
	il faudra voter non le 26 octobre prochain)

	Voici un exemple:

		L'exemple suivant est purement fictif et toute
		ressemblance avec la ralit est purement
		le fruit du hasard.

		Dans une compagnie fictive MSK, il y a trois
		programmeurs: Woodbang dit mosart, Jambon dit bambin,
		et louis dit le casseur de rseau.

		Tous trois sont occups  amliorer la version 4.2
		du programme 69/69. Et soudain avant mme que l'automne
		ne fut venue, chacun fait une livraison. Et il continue
		ensuite  travailler dans leur version respective.

		Woodbang livre la version 4.2.4.1
		Jambon livre la version 4.2.3.1
		Louis livre la version 4.2.6.1

		Pierrot le loup fait une intgration  partir des
		versions 4.2 4.2.3.1 4.2.4.1 4.2.6.1. Cela donne
		la version 4.3. Elle se compose de:

			Majoritairement des fichiers de 4.2.
			Des fichiers provenant d'une des 3 rvisions.
			Des fichiers provenant de la combinaison
				des 3 rvisions.

		Lorsque Woodbang dcide de travailler sur la
		version 4.3 et ainsi profiter des nouvelles options
		auto-reboot de louis le casseur et Jambon, quelques minutes
		se sont pass depuis qu'il a livr sa version 4.2.4.1. Entre
		temps, il a ajout 25 nouvelles options au programme. Ces
		options ont t ajouts  la rvision 4.2.4.1. Il ne peut
		pas simplement copier les fichiers modifis dans sa copie
		de la version 4.3.

	La rvision dans laquelle il a continu  travailler est
	la rvision "temporaire". La rvision dans laquelle il
	souhaite incorporer ses changement est la rvision destination.

*/
#

/*
	Enregistre conditionnellement les transactions pour faire l'intgration
	d'un fichier.
*/
static void ufusion_settrans(
	INTEGRE_DAT &intg,
	USERINFO *userref,
	MAKEFILE &mkfref,		// Rfrence commune approxim.
	USERINFO *usersrc,
	MAKEFILE &mkfsrc,		// Provenance immdiate du fichier rvis.
	USERINFO *userdst,
	MAKEFILE &mkfdst,		// Destination de l'intgration.
	const char *dirsrc,		// Path du rpertoire projet qui contient
							// fichier.
	const char *fichier,
	int efface)				// Ce fichier a t effac de la rvision source
{
	/* #Spcification: fusion / espace usager / rvision bidon
		Les fichiers qui sont dans l'espace usager n'ont pas
		encore t livrs. On va leur attribu une rvision
		bidon facile  reconnaitre lors de l'intgration.

		On placera simplement comme rvision dans integre.dat
		le numro 1000.0.0.0. Le module d'dition de la fusion fera
		un cas spcial pour ce numro de rvision.
	*/
	REVISION spcrev;	// Le constructeur assure 
						// spcrev.isnull() != 0

	const REVISION *revroot_src = usersrc->getrevsrc();
	const REVISION *revroot_dst = userdst->getrevsrc();

	MAKEFILE_FILE *fsrc = mkfsrc.locate(fichier);
	MAKEFILE_FILE *fdst = mkfdst.locate(fichier);
	MAKEFILE_FILE *fref = mkfref.locate(fichier);
	char srcpath[MAXSIZ_PATH]; // Path du fichier non livr
	path_make (dirsrc,fichier,srcpath);
	if (fsrc != NULL
		&& fdst != NULL
		&& fsrc->getrev()->cmp(fdst->getrev()) == 0){
		// ICI on est certain que la rvision usersrc est l'anctre
		// de userdst et videmment de la rvision  fusionner.
		char pathref[MAXSIZ_PATH];
		fsrc->getombpath(usersrc,pathref);
		FILE_REV_SPEC ref;
		ref.fichier  = fsrc->getrev();
		ref.dir = mkfsrc.getrev();
		ref.root = revroot_src;
		/* #Spcification: Fusion / Espace usager / automatique
			Pour importer sans intervention un fichier
			d'une rvision temporaire vers la rvision
			destination, il faut que les deux
			rvisions possdent la mme version de ce fichier.
		*/
		FILE_REV_SPEC rev;
		rev.fichier  = &spcrev;
		rev.dir = ref.dir;
		rev.root = ref.root;
		intg.addtrans(fichier,ref
			,pathref,efface ? '-' : '+'
			,rev,srcpath);
	}else{
		// ICI, on prend userref comme rvision anctre. C'est une
		// approximation. Se referer  la spec plus bas pour comprendre
		// les noms des variables ref, tmp et dst
		FILE_REV_SPEC ref;
		ref.fichier = NULL;
		ref.dir = mkfref.getrev();
		ref.root = userref->getrevsrc();
		FILE_REV_SPEC tmp;
		tmp.fichier = NULL;
		tmp.dir = mkfsrc.getrev();
		tmp.root = revroot_src;
		FILE_REV_SPEC dst;
		dst.fichier = NULL;
		dst.dir = mkfdst.getrev();
		dst.root = revroot_dst;

		/* #Spcification: Fusion / Espace usager / manuelle
			Voici un exemple, o une fusion interactive est requise.
			Soit le fichier A et ses diffrentes rvisions:

			#
			      ---> 4.2.3.1 -------
				 |				      |--> 4.3 .....................
			4.2--|                 ---	   (Rvision destination)   .
				 |				  |									.
				  ---> 4.2.4.1 -->|									.
								   --- Rvision temporaire	........
									   (4.2.4.2)
			4.2 est la rvision rfrence
			#
			Pour simplifier la fusion, l'usager doit
			pouvoir visualiser les diffrences suivantes:
			#
				4.2.4.1 vs 4.2
				4.2.3.1 vs 4.2
				4.3     vs 4.2
				4.2.4.2 vs 4.2.4.1 (Probablement
									la plus importante).
			#

			La version 4.3 a t construites manuellement  partir
			de diffrentes versions (4.1.3.1 et 4.1.4.1). De plus
			l'intgrateur a pu ajout d'autre chose (l'ivresse du moment).
			La provenance de la version 4.3 ne peut pas tre dmontr absoluement.
			C'est pourquoi, il faut permettrent  peu prs toutes les
			combinaisons de diffrences imaginables.
		*/
		char *pathref = NULL;
		char tmppath[MAXSIZ_PATH];
		if (fref != NULL){
			ref.fichier = fref->getrev();
			fref->getombpath(userref,tmppath);
			pathref = tmppath;
		}
		if (efface){
			intg.addtrans(fichier,ref,pathref
				,'-'
				,tmp,srcpath);
		}else{
			tmp.fichier = &spcrev;
			intg.addtrans(fichier,ref,pathref
				,fsrc == NULL && fdst == NULL ? '?' : '+'
				,tmp,srcpath);
		}
		if (fsrc != NULL){
			char tmppath[MAXSIZ_PATH];
			fsrc->getombpath(usersrc,tmppath);
			tmp.fichier = fsrc->getrev();
			intg.addtrans (fichier,ref
				,pathref,'+'
				,tmp,tmppath);
		}else{
			intg.addtrans (fichier,ref
				,pathref,'-'
				,tmp,NULL);
		}
		if (fdst != NULL){
			char tmppath[MAXSIZ_PATH];
			fdst->getombpath(userdst,tmppath);
			dst.fichier = fdst->getrev();
			intg.addtrans (fichier,ref
				,pathref,'+'
				,dst,tmppath);
		}else{
			intg.addtrans (fichier,ref
				,pathref,'-'
				,dst,NULL);
		}
	}
}

/*
	Balaye les deux rpertoires rcursivement et construit un integre.dat
	dans chaque rpertoire. Ces integre.dat agira comme guide pour
	la fusion. Mme format que pour les fusions dans /kit/build.
*/
static int ufusion_scan (
	USERINFO *usersrc,		// Environnement  intgrer dans userdst
	USERINFO *userdst,
	USERINFO *userref,		// Racine des deux environnement prcdant
	PRJCTRL *ctrl,
	const char *projet,
	PROJET_LOG *log,
	MAKEFILE *mkrefanc,		// makefile.dat du projet anctre de celui-ci
							// obtenue  partir de userref.
							// == NULL si projet == ""
	MAKEFILE *mksrcanc)		// makefile.dat du projet anctre de celui-ci
							// obtenue  partir de usersrc.
							// == NULL si projet == ""
	
{
	char dirdst[MAXSIZ_PATH];
	userdst->makusrpath(projet,dirdst);
	/* #Spcification: fusion / Espace usager / slection des rpertoires.
		La fusion est faite uniquement pour les projets que l'usager
		a import pour la rvision cible.
	*/
	if (file_type(dirdst)==1){
		char dirsrc[MAXSIZ_PATH];
		usersrc->makusrpath(projet,dirsrc);
		SSTRINGS tb;
		int nb = ctrl->dirsource (dirsrc,WILD_ALLFILE,tb);
		log->printf ("Rpertoire %s [%d]\n",dirsrc,nb);
		if (nb > 0){
			/* #Spcification: fusion / espace usager / rvision rfrence
				La rvision rfrence n'est probablement pas
				disponible dans l'espace usager. Pour un
				projet ou sous-projet, on doit retrouver la rvision
				correspondante. En effet la rvision X est une combinaison
				de rvisions diffrentes de sous-projet.
			*/
			const REVISION *revision_ref = userref->getrevsrc();
			const REVISION *revision_src = usersrc->getrevsrc();
			if (projet[0] != '\0'){
				// Sous-projet, il faut consulter mkrefanc et mksrcanc pour
				// dterminer la bonne rvision rfrence et source.
				char nom[MAXSIZ_NAME*2];
				path_splitlex (projet,NULL,nom);
				strcat (nom,"/makefile.dat");
				MAKEFILE_FILE *file = mkrefanc->locate(nom);
				if (file == NULL){
					// Nouveau rpertoire.
					log->printf ("Nouveau rpertoire %s\n",projet);
					/* #Spcification: fusion / espace usager / nouveaux rpertoires
						Le traitement  raliser n'est pas clair
						dans le cas ou un nouveau rpertoire ou projet
						a t cr. En ce moment, on signale simplement
						 l'usager qui devra fusionner manuellement le
						rpertoire en question.
					*/
					// On laisse revision_ref comme il est.
					// En utilisant la rvision de la racine
					// ca va charger un makefile.dat vide.
				}else{
					revision_ref = file->getrev();
				}
				// Mme traitement pour la rvision source
				file = mksrcanc->locate(nom);
				if (file == NULL){
					// Nouveau rpertoire.
					log->printf ("Nouveau rpertoire %s : environnement tmp\n",projet);
					// On laisse revision_src comme il est.
					// En utilisant la rvision de la racine
					// ca va charger un makefile.dat vide.
				}else{
					revision_src = file->getrev();
				}
			}
			MAKEFILE mkfref(NULL,true,revision_ref,projet,userref);
			// Pour le rpertoire source, on obtient le makefile.dat
			// de /kit/ombre, pour eviter le makefile.del locale.
			// Voir MAKEFILE::MAKEFILE
			MAKEFILE mkfsrc(NULL,1,revision_src,projet,usersrc);
			// Scan les sous-rpertoires
			for (int i=0; i<nb; i++){
				const char *fichier = tb.getitem(i)->get();
				if (ctrl->isdir(fichier)){
					char sub_projet[MAXSIZ_PATH];
					path_make (projet,fichier,sub_projet);
					path_stripsep(sub_projet,sub_projet);
					ufusion_scan (usersrc,userdst,userref,ctrl
						,sub_projet,log,&mkfref,&mkfsrc);
				}
			}
			MAKEFILE mkfdst(NULL,true,projet,userdst);
			INTEGRE_DAT intg(dirdst);
			for (int i=0; i<nb; i++){
				const char *fichier = tb.getitem(i)->get();
				if (!ctrl->isdir(fichier)){
					ufusion_settrans (intg,userref,mkfref,usersrc,mkfsrc
						,userdst,mkfdst,dirsrc,fichier,0);
				}
			}
			/* #Spcification: Fusion / espace usager / makefile.del
				Les fichiers makefile.del sont interprter lors de la
				fusion et prsenter interactivement lors de l'dition
				de la fusion.
			*/
			char delpath[MAXSIZ_PATH]; // Path du fichier makefile.del
			path_make (dirsrc,"makefile.del",delpath);
			FILE *fdel = fopen (delpath,"r");
			// Le fichier makefile.del est optionnel.
			if (fdel != NULL){
				char buf[MAXSIZ_PATH];
				while (fgets(buf,sizeof(buf)-1,fdel)!=NULL){
					str_strip (buf,buf);
					if (buf[0] != '\0'){
						ufusion_settrans (intg,userref,mkfref,usersrc,mkfsrc
							,userdst,mkfdst,dirsrc,buf,1);
					}
				}
				fclose (fdel);
			}			
		}
	}
	return 0;
}
/*
	Slectionne deux rvisions  fusionner dans l'environnement usager
	et produit les fichiers integre.dat pilotant la fusion interactive
	de integre_do().
*/
void ufusion_prepare (USERINFO *user)
{
	char home[MAXSIZ_PATH];
	user->gethome(home);
	char *tbrev[100];
	int nb = projet_getrevdir (home,tbrev,100);
	if (nb < 2){
		xconf_error (MSG_U(E_MIN2PRJ
			,"Vous devez avoir au moins deux rvisions"));
	}else{
		DIALOG dia;
		char src = 0;
		char dst = 1;
		dia.newf_head ("",MSG_U(H_FUSIONSIMPLE
			,"Tmp\tDst\tprog/Version/rpertoire/source"));
		for (int i=0; i<nb; i++){
			char revstr[MAXSIZ_REVISION];
			revision_path2rev (tbrev[i],revstr);
			#if 0
			dia.newf_radio (&src,i,i==0);
			page_setupradioc (&page,-1,&dst,i,i==1);
			page_setupfixestr0 (&page,-1,revstr,MAXSIZ_REVISION);
			#endif
		}
		int nof =0;
		MENU_STATUS code = dia.edit (MSG_U(T_FUSIONSIMPLE,"Fusion simple")
			,""
			,help_nil
			,nof);
		if (code == MENU_ACCEPT){
			USERINFO usersrc(user);
			char verstr[MAXSIZ_REVISION];
			revision_path2rev(tbrev[src],verstr);
			usersrc.setversion(verstr);
			/* #Spcification: fusion / espace usager / rvision rfrence
				La rvision rfrence (commune  la rvision temporaire
				et la destination) est la version d'intgration
				de la rvision temporaire.
			
					Rvision temporaire : 4.2.4.1
					Rvision destination: 4.3
					Rvision rfrence  : 4.2

				C'est un approximation. Il n'est pas toujours possible
				de dterminer la rvision rfrence (anctre) exacte
				de deux rvision.
			*/
			REVISION rev(verstr);
			rev.cnv (REVISION_INTEGRE);
			rev.format (verstr);
			USERINFO userref(user);
			userref.setversion(verstr);
			USERINFO userdst(user);
			revision_path2rev(tbrev[dst],verstr);
			userdst.setversion(verstr);
			PRJCTRL ctrl (user);
			WINDOW_LOG log(NULL,1);
			log.show();
			ufusion_scan(&usersrc,&userdst,&userref,&ctrl,"",&log
				,NULL,NULL);
		}
	}
}
